#include "stdaza.h"
#include "input.h"

static HGLRC hRC;		
static HDC hDC;		

HWND *thWnd;

static	PIXELFORMATDESCRIPTOR pfd=
{
	sizeof(PIXELFORMATDESCRIPTOR),		
	1,									
	PFD_DRAW_TO_WINDOW |				
	PFD_SUPPORT_OPENGL |				
	PFD_DOUBLEBUFFER,					
	PFD_TYPE_RGBA,						
	16,									
	0, 0, 0, 0, 0, 0,					
	0,									
	0,									
	0,									
	0, 0, 0, 0,							
	16,									
	0,									
	0,									
	PFD_MAIN_PLANE,						
	0,									
	0, 0, 0								
};



void swapBuffers()
{
	SwapBuffers(hDC);
}

LRESULT CALLBACK WndProc(	HWND	hWnd,
				UINT	message,
				WPARAM	wParam,
				LPARAM	lParam)
{
	
	switch (message)						
	{
	case WM_KEYDOWN:
	case WM_COMMAND:
	

		case WM_DESTROY:
		case WM_CLOSE:
			closeWindow();
			break;

		
		default:
			return (DefWindowProc(hWnd, message, wParam, lParam));
	}
return (0);
}

void initWindow(HWND hWnd)
{
	thWnd = &hWnd;
	int PixelFormat;

	hDC = GetDC(hWnd);				
	PixelFormat = ChoosePixelFormat(hDC, &pfd);		

	if (!PixelFormat)
	{
		MessageBox(0,"Can't Find A Suitable PixelFormat.","Error",MB_OK|MB_ICONERROR);
		closeWindow();
	}

	if(!SetPixelFormat(hDC,PixelFormat,&pfd))
	{
		MessageBox(0,"Can't Set The PixelFormat.","Error",MB_OK|MB_ICONERROR);
		closeWindow();
	}

	hRC = wglCreateContext(hDC);
	if(!hRC)
	{
		MessageBox(0,"Can't Create A GL Rendering Context.","Error",MB_OK|MB_ICONERROR);
		closeWindow();
	}

	if(!wglMakeCurrent(hDC, hRC))
	{
		MessageBox(0,"Can't activate GLRC.","Error",MB_OK|MB_ICONERROR);
		closeWindow();
	}
}

void closeWindow()
{
	//closeInput();

	ChangeDisplaySettings(NULL, 0);

	wglMakeCurrent(hDC,NULL);
	wglDeleteContext(hRC);
	ReleaseDC(*thWnd,hDC);
	exit(0);
}

int createWindowsApp(	HINSTANCE	hInstance, 
						HINSTANCE	hPrevInstance, 
						LPSTR		lpCmdLine, 
						int			nCmdShow,
						HWND		*hWnd)
{
	WNDCLASS	wc;	
	wc.style			= CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
	wc.lpfnWndProc		= (WNDPROC) WndProc;
	wc.cbClsExtra		= 0;
	wc.cbWndExtra		= 0;
	wc.hInstance		= hInstance;
	wc.hIcon			= NULL;
	wc.hCursor			= LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground	= NULL;
	wc.lpszMenuName		= NULL;
	wc.lpszClassName	= "OpenGL WinClass";

	if(!RegisterClass(&wc))
	{
		MessageBox(0,"Failed To Register The Window Class.","Error",MB_OK|MB_ICONERROR);
		return FALSE;
	}

	*hWnd = CreateWindow(
	"OpenGL WinClass",
	"The Azaryth Chronicles",		

	//WS_POPUP
	//WS_CLIPCHILDREN |
	//WS_CLIPSIBLINGS
	WS_OVERLAPPED
	,


	0, 0,												
	640, 480,											

	NULL,
	NULL,
	hInstance,
	NULL);

	if(!hWnd)
	{
		MessageBox(0,"Window Creation Error.","Error",MB_OK|MB_ICONERROR);
		return FALSE;
	}

	initWindow(*hWnd);
	

	DEVMODE dmScreenSettings;											

	memset(&dmScreenSettings, 0, sizeof(DEVMODE));						
	dmScreenSettings.dmSize			= sizeof(DEVMODE);					
	dmScreenSettings.dmPelsWidth	= 640;								
	dmScreenSettings.dmPelsHeight	= 480;								
	dmScreenSettings.dmFields		= DM_PELSWIDTH | DM_PELSHEIGHT;		
	//ChangeDisplaySettings(&dmScreenSettings, CDS_FULLSCREEN);			

	ShowWindow(*hWnd, SW_SHOW);
	UpdateWindow(*hWnd);
	SetFocus(*hWnd);
	wglMakeCurrent(hDC,hRC);
	
	//initInput(hInstance,*hWnd);

	return TRUE;
}

void errorWindow(char* error)
{
	MessageBox(0,error,"Error",MB_OK|MB_ICONERROR);
}