#define DIRECTINPUT_VERSION 0x0700 
#include <dinput.h>
#include "input.h"
#include "init.h"

#define KEYDOWN(name,key) (name[key] & 0x80) 

HRESULT hr; 
LPDIRECTINPUT7 g_lpdi;  

LPDIRECTINPUTDEVICE7 diKeyboard;
LPDIRECTINPUTDEVICE7 diMouse;

char rawKB[256];
char clearKB[256];

int rawKBmode = 0;

DIMOUSESTATE diMS;
int sizeofMousePacket = sizeof(DIMOUSESTATE);

int mouseoffx,mouseoffy;

void initKeyboard(HWND hwnd)
{
	for (int x = 0; x < 256; x++)
	{
		clearKB[x] = 1;
	}

	hr = g_lpdi->CreateDeviceEx(GUID_SysKeyboard, IID_IDirectInputDevice7, (void**)&diKeyboard, NULL); 

	if FAILED(hr)
	{
		MessageBox(0,"DirectInput error creating keyboard device","Error",MB_OK|MB_ICONERROR);
		closeWindow();
	}

	hr = diKeyboard->SetDataFormat(&c_dfDIKeyboard); 

	if FAILED(hr)
	{
		MessageBox(0,"DirectInput error setting data format for keyboard","Error",MB_OK|MB_ICONERROR);
		closeWindow();
	}

	hr = diKeyboard->SetCooperativeLevel(hwnd, DISCL_FOREGROUND | DISCL_EXCLUSIVE); 

	if FAILED(hr)
	{
		MessageBox(0,"DirectInput error setting cooperation","Error",MB_OK|MB_ICONERROR);
		closeWindow();
	}

	diKeyboard->Acquire();

}

void initMouse(HWND hwnd)
{
	hr = g_lpdi->CreateDeviceEx(GUID_SysMouse, IID_IDirectInputDevice7, (void**)&diMouse, NULL);

	if FAILED(hr)
	{
		MessageBox(0,"DirectInput error getting mouse device","Error",MB_OK|MB_ICONERROR);
		closeWindow();
	}

	hr = diMouse->SetDataFormat(&c_dfDIMouse);

	if FAILED(hr)
	{
		MessageBox(0,"DirectInput error setting mouse format","Error",MB_OK|MB_ICONERROR);
		closeWindow();
	}		

	hr = diMouse->SetCooperativeLevel(hwnd, DISCL_NONEXCLUSIVE | DISCL_FOREGROUND);

	if FAILED(hr)
	{
		MessageBox(0,"DirectInput error setting mouse cooperation level","Error",MB_OK|MB_ICONERROR);
		closeWindow();
	}

	DIPROPDWORD dipdw;

    dipdw.diph.dwSize = sizeof(DIPROPDWORD);
    dipdw.diph.dwHeaderSize = sizeof(DIPROPHEADER);
    dipdw.diph.dwObj = 0;
    dipdw.diph.dwHow = DIPH_DEVICE;
    dipdw.dwData = DIPROPAXISMODE_ABS;

    diMouse->SetProperty(DIPROP_AXISMODE, &dipdw.diph);

	diMouse->Acquire();

	diMouse->GetDeviceState(sizeofMousePacket,&diMS);
	mouseoffx = diMS.lX * -1;
	mouseoffy = diMS.lY * -1;
}

void initInput(HINSTANCE hinst, HWND hwnd)
{
	hr = DirectInputCreateEx(hinst, DIRECTINPUT_VERSION, IID_IDirectInput7A, (void**)&g_lpdi, NULL);

	if FAILED(hr)
	{
		MessageBox(0,"DirectInput error initializing.","Error",MB_OK|MB_ICONERROR);
		closeWindow();
	}
	
	initKeyboard(hwnd);
	initMouse(hwnd);
}


void closeInput()
{
	if (g_lpdi)     
	{         
		if (diKeyboard)         
		{ 
            diKeyboard->Unacquire();
			diKeyboard->Release();
            diKeyboard = NULL;
		}         

		if (diMouse)
		{
			diMouse->Unacquire();
			diMouse->Release();
			diMouse = NULL;
		}

		g_lpdi->Release();
        g_lpdi = NULL;     
	} 
}

void setRawKBMode(int mode)
{
	rawKBmode = mode;
}

void updateKeyboard()
{
	diKeyboard->GetDeviceState(256,(LPVOID)&rawKB);

	if (!rawKBmode)
	{
		for (int x = 0; x < 256; x++)
		{
			if (rawKB[x] & 0x80) clearKB[x] = 1;
			if (KEYDOWN(rawKB,x) && (clearKB[x] == 0)) rawKB[x] = rawKB[x] & 0x7F;
		}
	}
}

int keyDown(int key)
{
	clearKB[key] = 0;
	return KEYDOWN(rawKB,key);
}	

void updateDI()
{
	updateKeyboard();
}

mouseInfo* getMouseInfo()
{
	mouseInfo* tempMI = new mouseInfo();

	diMouse->GetDeviceState(sizeofMousePacket,&diMS);

	tempMI->x = diMS.lX + mouseoffx;
	tempMI->y = diMS.lY + mouseoffy;
	if (diMS.rgbButtons[0] & 0x80) tempMI->b1 = 1; else tempMI->b1 = 0;
	if (diMS.rgbButtons[1] & 0x80) tempMI->b2 = 1; else tempMI->b2 = 0;

	return tempMI;
}