#ifndef __land_h
#define __land_h

#include "stdaza.h"
#include "tex.h"

#define TEX_LAND_GRASS 0
#define TEX_LAND_PLAIN 1
#define TEX_LAND_DIRT 2
#define TEX_LAND_STONE 3
#define TEX_LAND_SNOW 4
#define TEX_LAND_SWAMP 5
#define TEX_LAND_SAND 6

#define TEX_LAND_COUNT 7


#define TEX_TRANS_LIGHT 0
#define TEX_TRANS_HEAVY 1
#define TEX_TRANS_STRIP 2

#define TEX_TRANS_COUNT 3

#define LAND_CLIP_LIMIT 4

#define TEX_LAND_STRETCH 16.0
#define TEX_STRETCH_RATIO (1.0 / TEX_LAND_STRETCH)

struct vertexData {
	float height;
	float shade;

	float trans1;
	float trans2;
};

struct faceData {
	int landtype;
	int landtex;
	int transtex1;
	int transtex2;
};

class Landscape
{
private:

	Texture* landtex[TEX_LAND_COUNT];
	Texture* transtex[TEX_TRANS_COUNT];

	int size, sizeBits;

	int numfaces, numverts;

	vertexData* verts;
	faceData* faces;

	double modelMatrix[16];
	double projMatrix[16];
	int viewport[4];

public:

	Landscape(int size, int sizeBits);
	~Landscape();

	void clear();

	void generateHeights(int, float);
	void smoothHeightEdges(int);
	void generateShades();
	void generate();

	void drawMesh();
	void drawPlainSolid();
	void drawLand();
	void drawRegion(int x, int y, int s);
	void clipRegion(int x, int y, int s);

	vertexData* vert(int x, int y);
	faceData* face(int x, int y);

	void updateViewMatricies();
};



#endif